package com.agiledeveloper.annotations;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.tools.Diagnostic;
import java.util.Set;

@SupportedSourceVersion(SourceVersion.RELEASE_16)
@SupportedAnnotationTypes("com.agiledeveloper.annotations.Immutable")
public class ImmutableAnnotationProcessor extends AbstractProcessor {
  @Override
  public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
    if(
    annotations.stream()
      .flatMap(annotation -> roundEnv.getElementsAnnotatedWith(annotation).stream())
      .flatMap(element -> element.getEnclosedElements().stream())
      .filter(element -> element.getKind() == ElementKind.FIELD)
      .filter(element ->
        element.getModifiers()
          .contains(Modifier.FINAL) ? false : true)
      .count() > 0) {
      processingEnv.getMessager().printMessage(Diagnostic.Kind.ERROR, "At least one field is non final");
    }

    return false;
  }
}
